# This file was auto-generated by ZETUP
#
# Zimmermann's Extensible Tools for Unified Project setups
#
# https://github.com/zimmermanncode/zetup


from __future__ import absolute_import

import sys
import os
from collections import OrderedDict

from zetup.version import Version
from zetup.dist import Distribution
from zetup.requires import Requirements
from zetup.extras import Extras
from zetup.package import Packages, Package
from zetup.notebook import Notebook



zfg = sys.modules[__name__]

NAME = 'zetup'

LICENSE = 'LGPLv3'

TITLE = 'zetup'

PYTHON = ['2.7', '3.3', '3.4', '3.5', '3.6']

MODULES = []

FORCE_MAKE = True

KEYWORDS = ['zetup', 'project', 'tools', 'config', 'ini', 'setup', 'requirements', 'extras', 'packages', 'package', 'toplevel', 'object', 'meta', 'subprocess', 'process', 'popen', 'call', 'python3', 'python3']

PACKAGES = Packages([

Package('zetup',
  sources=[
    '__init__.py',
    '__main__.py',
    'annotate.py',
    'classpackage.py',
    'conda.py',
    'config.py',
    'dist.py',
    'error.py',
    'extras.py',
    'modules.py',
    'object.py',
    'package.py',
    'requires.py',
    'resolve.py',
    'script.py',
    'test_classpackage.py',
    'version.py',
    'zetup.py'
    ],
  subpackages=[
    
Package('zetup.commands',
  sources=[
    '__init__.py',
    'command.py',
    'conda.py',
    'del_.py',
    'dev.py',
    'error.py',
    'pytest.py',
    'run.py',
    'test.py',
    'tox.py'
    ],
  subpackages=[
    
Package('zetup.commands.make',
  sources=[
    '__init__.py'
    ],
  subpackages=[
    
    ],
  )

    ],
  )
,
    
Package('zetup.doc',
  sources=[
    '__init__.py'
    ],
  subpackages=[
    
    ],
  )
,
    
Package('zetup.notebook',
  sources=[
    '__init__.py',
    'jinja.py'
    ],
  subpackages=[
    
    ],
  )
,
    
Package('zetup.process',
  sources=[
    '__init__.py',
    'scons.py'
    ],
  subpackages=[
    
    ],
  )

    ],
  )

  ], root=os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

EMAIL = 'user@zimmermann.co'

AUTHOR = 'Stefan Zimmermann'

EXTRAS = Extras([
('commands', Requirements("""
path.py >= 10.3 #import path
jinjatools >= 0.1.7

""")),
('notebook', Requirements("""
path.py >= 10.3 #import path
nbconvert >= 5.2

""")),
('pytest', Requirements("""
pytest

"""))
], zfg=zfg)

REQUIRES = Requirements("""
# setuptools
setuptools_scm >= 1.15

""", zfg=zfg)

URL = 'https://github.com/zimmermanncode/zetup'

SETUP_HOOKS = []

VERSION = Version('0.2.42')

KEEP_MADE = []

NO_MAKE = ['setup.py', 'tox.ini']

TEST_COMMANDS = ['py.test -v test']

DESCRIPTION = "Zimmermann's Extensible Tools for Unified Project_setups"

SETUP_REQUIRES = None

in_repo = True

SCRIPTS = {'zetup': 'zetup.script:run'}

SETUP_KEYWORDS = {'use_zetup': 'zetup:setup_entry_point'}

DISTRIBUTION = Distribution(zfg)

CLASSIFIERS = ['Development Status :: 3 - Alpha', 'Intended Audience :: Developers', 'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', 'Operating System :: OS Independent', 'Topic :: Software Development', 'Topic :: Utilities', 'Programming Language :: Python', 'Programming Language :: Python :: 2.7', 'Programming Language :: Python :: 3.3', 'Programming Language :: Python :: 3.4', 'Programming Language :: Python :: 3.5', 'Programming Language :: Python :: 3.6']